@@ -2,19 +2,20 @@
 /*  GMime
  *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -150,10 +151,10 @@ message_partial_set_content_type (GMimeObject *object, GMimeContentType *content
 	partial->id = g_strdup (value);
 	
 	value = g_mime_content_type_get_parameter (content_type, "number");
-	partial->number = value ? atoi (value) : -1;
+	partial->number = value ? strtol (value, NULL, 10) : -1;
 	
 	value = g_mime_content_type_get_parameter (content_type, "total");
-	partial->total = value ? atoi (value) : -1;
+	partial->total = value ? strtol (value, NULL, 10) : -1;
 	
 	GMIME_OBJECT_CLASS (parent_class)->set_content_type (object, content_type);
 }
@@ -176,7 +177,7 @@ g_mime_message_partial_new (const char *id, int number, int total)
 	GMimeContentType *type;
 	char *num;
 	
-	partial = g_object_new (GMIME_TYPE_MESSAGE_PARTIAL, NULL, NULL);
+	partial = g_object_new (GMIME_TYPE_MESSAGE_PARTIAL, NULL);
 	
 	type = g_mime_content_type_new ("message", "partial");
 	
@@ -290,13 +291,11 @@ g_mime_message_partial_reconstruct_message (GMimeMessagePartial **partials, size
 	
 	g_return_val_if_fail (num > 0, NULL);
 	
-	id = g_mime_message_partial_get_id (partials[0]);
-	if (!id)
+	if (!(id = g_mime_message_partial_get_id (partials[0])))
 		return NULL;
 	
 	/* get them into the correct order... */
-	qsort ((void *) partials, num, sizeof (gpointer),
-	       partial_compare);
+	qsort ((void *) partials, num, sizeof (gpointer), partial_compare);
 	
 	/* only the last message/partial part is REQUIRED to have the total parameter */
 	total = g_mime_message_partial_get_total (partials[num - 1]);
